from __future__ import unicode_literals

import logging

from django.conf import settings
from django.db import models
from django.db.models.signals import post_save, pre_delete

from django.contrib.algoliasearch.models import AlgoliaIndex
from django.contrib.algoliasearch.version import VERSION

from algoliasearch import algoliasearch

logger = logging.getLogger(__name__)


class AlgoliaEngineError(Exception):
    '''Something went wrong with Algolia engine.'''


class RegistrationError(AlgoliaEngineError):
    '''Something went wrong when registering a model with the search sengine.'''


class AlgoliaEngine(object):
    def __init__(self, app_id=None, api_key=None):
        '''Initializes Algolia engine.'''
        if not (app_id and api_key):
            params = getattr(settings, 'ALGOLIA', None)
            if params:
                app_id = params['APPLICATION_ID']
                api_key = params['API_KEY']
            else:
                # @Deprecated: 1.1.0
                app_id = settings.ALGOLIA_APPLICATION_ID
                api_key = setting.ALGOLIA_API_KEY

        self.__registered_models = {}
        self.client = algoliasearch.Client(app_id, api_key)
        self.client.set_extra_header('User-Agent',
                                     'Algolia for Django {}'.format(VERSION))

    def is_registered(self, model):
        '''Checks whether the given models is registered with Algolia engine.'''
        return model in self.__registered_models

    def register(self, model, index_cls=AlgoliaIndex):
        '''
        Registers the given model with Algolia engine.

        If the given model is already registered with Algolia engine, a
        RegistrationError will be raised.
        '''
        # Check for existing registration
        if self.is_registered(model):
            raise RegistrationError(
                '{} is already registered with Algolia engine'.format(model))
        # Perform the registration.
        index_obj = index_cls(model, self.client)
        self.__registered_models[model] = index_obj
        # Connect to the signalling framework.
        post_save.connect(self.__post_save_receiver, model)
        pre_delete.connect(self.__pre_delete_receiver, model)
        logger.info('REGISTER %s', model)

    def unregister(self, model):
        '''
        Unregisters the given model with Algolia engine.

        If the given model is not registered with Algolia engine, a
        RegistrationError will be raised.
        '''
        if not self.is_registered(model):
            raise RegistrationError(
                '{} is not registered with Algolia engine'.format(model))
        # Perform the unregistration.
        del self.__registered_models[model]
        # Disconnect fron the signalling framework.
        post_save.disconnect(self.__post_save_receiver, model)
        pre_delete.disconnect(self.__pre_delete_receiver, model)
        logger.info('UNREGISTER %s', model)

    def get_registered_models(self):
        '''Returns a sequence of models that have been registered with Algolia engine.'''
        return list(self.__registered_models.keys())

    def get_adapter(self, model):
        '''Returns the adapter associated with the given model.'''
        if self.is_registered(model):
            return self.__registered_models[model]
        raise RegistrationError(
            '{} is not registered with Algolia engine'.format(model))

    def get_adapter_from_instance(self, instance):
        model = instance.__class__
        return self.get_adapter(model)

    def update_obj_index(self, obj):
        adapter = self.get_adapter_from_instance(obj)
        adapter.update_obj_index(obj)

    def delete_obj_index(self, obj):
        adapter = self.get_adapter_from_instance(obj)
        adapter.delete_obj_index(obj)

    def raw_search(self, model, query='', params={}):
        adapter = self.get_adapter(model)
        return adapater.raw_search(query, params)

    # Signalling hooks.

    def __post_save_receiver(self, instance, **kwargs):
        '''Signal handler for when a registered model has been saved.'''
        logger.debug('RECEIVE post_save FOR %s', instance.__class__)
        self.update_obj_index(instance)

    def __pre_delete_receiver(self, instance, **kwargs):
        '''Signal handler for when a registered model has been deleted.'''
        logger.debug('RECEIVE pre_delete FOR %s', instance.__class__)
        self.delete_obj_index(instance)

# Algolia engine
algolia_engine = AlgoliaEngine()



'''
AlgoliaSearch integration for Django.
http://www.algolia.com
'''

from __future__ import unicode_literals

from django.contrib.algoliasearch.models import AlgoliaIndex
from django.contrib.algoliasearch.registration import AlgoliaEngine
from django.contrib.algoliasearch.registration import algolia_engine

algolia_engine = algolia_engine
register = algolia_engine.register
unregister = algolia_engine.unregister
get_registered_model = algolia_engine.get_registered_models
get_adapter = algolia_engine.get_adapter
raw_search = algolia_engine.raw_search



from django.core.management.base import BaseCommand
from django.contrib import algoliasearch


class Command(BaseCommand):
    help = 'Clear index.'

    def add_arguments(self, parser):
        parser.add_argument('--model', nargs='+', type=str)

    def handle(self, *args, **options):
        '''Run the management command.'''
        self.stdout.write('Clear index:')
        for model in algoliasearch.get_registered_model():
            adapter = algoliasearch.get_adapter(model)
            if options.get('model', None) and not (model.__name__ in options['model']):
                continue

            adapter.clear_index()
            self.stdout.write('\t* {}'.format(model.__name__))



from __future__ import unicode_literals

import logging

from django.conf import settings
from django.db import models

from algoliasearch import algoliasearch

logger = logging.getLogger(__name__)


class AlgoliaIndexError(Exception):
    '''Something went wrong with an Algolia Index.'''


class AlgoliaIndex(object):
    '''An index in the Algolia backend.'''

    # Use to specify a custom field that will be used for the objectID.
    # This field should be unique.
    custom_objectID = None

    # Use to specify the fields that should be included in the index.
    fields = ()

    # Use to specify the geo-fields that should be used for location search.
    # The attribute should be a callable that returns a tuple.
    geo_field = None

    # Use to specify the field that should be used for filtering by tag.
    tags = None

    # Use to specify the index to target on Algolia.
    index_name = None

    # Use to specify the settings of the index.
    settings = {}

    # Use to specify a callable that say if the instance should be indexed.
    # The attribute should be a callable that returns a boolean.
    should_index = None

    # Instance of the index from algoliasearch client
    __index = None

    def __init__(self, model, client):
        '''Initializes the index.'''
        self.model = model
        self.__client = client
        self.__set_index(client)

        all_fields = model._meta.get_all_field_names()

        # Avoid error when there is only one field to index
        if isinstance(self.fields, str):
            self.fields = (self.fields, )

        # Check fields
        for field in self.fields:
            if not (hasattr(model, field) or (field in all_fields)):
                raise AlgoliaIndexError(
                    '{} is not an attribute of {}.'.format(field, model))

        # If no fields are specified, index all the fields of the model
        if not self.fields:
            self.fields = all_fields
            self.fields.remove('id')

        # Check custom_objectID
        if self.custom_objectID:
            if not (hasattr(model, self.custom_objectID) or
                    (self.custom_objectID in all_fields)):
                raise AlgoliaIndexError('{} is not an attribute of {}.'.format(
                    self.custom_objectID, model))

        # Check tags
        if self.tags:
            if not (hasattr(model, self.tags) or (self.tags in all_fields)):
                raise AlgoliaIndexError('{} is not an attribute of {}'.format(
                    self.tags, model))

        # Check geo_field
        if self.geo_field:
            if hasattr(model, self.geo_field):
                attr = getattr(model, self.geo_field)
                if not callable(attr):
                    raise AlgoliaIndexError('{} should be a callable.'.format(
                        self.geo_field))
            else:
                raise AlgoliaIndexError('{} is not an attribute of {}.'.format(
                    self.geo_field, model))

        if self.should_index:
            if hasattr(model, self.should_index):
                attr = getattr(model, self.should_index)
                if not callable(attr):
                    raise AlgoliaIndexError('{} should be a callable.'.format(
                        self.should_index))
            else:
                raise AlgoliaIndexError('{} is not an attribute of {}.'.format(
                    self.should_index, model))

    def __set_index(self, client):
        '''Get an instance of Algolia Index'''
        params = getattr(settings, 'ALGOLIA', None)

        if not self.index_name:
            self.index_name = self.model.__name__

        if params:
            if 'INDEX_PREFIX' in params:
                self.index_name = params['INDEX_PREFIX'] + '_' + self.index_name
            if 'INDEX_SUFFIX' in params:
                self.index_name += '_' + params['INDEX_SUFFIX']
        else:
            # @Deprecated: 1.1.0
            if hasattr(settings, 'ALGOLIA_INDEX_PREFIX'):
                self.index_name = settings.ALGOLIA_INDEX_PREFIX + '_' + self.index_name
            if hasattr(settings, 'ALGOLIA_INDEX_SUFFIX'):
                self.index_name += '_' + settings.ALGOLIA_INDEX_SUFFIX

        self.__index = client.init_index(self.index_name)
        self.__tmp_index = client.init_index(self.index_name + '_tmp')

    def __get_objectID(self, instance):
        '''Return the objectID of an instance.'''
        if self.custom_objectID:
            attr = getattr(instance, self.custom_objectID)
            if callable(attr):
                attr = attr()
            return attr
        else:
            return instance.pk

    def _build_object(self, instance):
        '''Build the JSON object.'''
        tmp = {'objectID': self.__get_objectID(instance)}
        if isinstance(self.fields, dict):
            for key, value in self.fields.items():
                attr = getattr(instance, key)
                if callable(attr):
                    attr = attr()
                tmp[value] = attr
        else:
            for field in self.fields:
                attr = getattr(instance, field)
                if callable(attr):
                    attr = attr()
                tmp[field] = attr

        if self.geo_field:
            attr = getattr(instance, self.geo_field)
            if callable(attr):
                attr = attr()
            tmp['_geoloc'] = {'lat': attr[0], 'lng': attr[1]}

        if self.tags:
            attr = getattr(instance, self.tags)
            if callable(attr):
                attr = attr()
            if not isinstance(attr, list):
                attr = list(attr)
            tmp['_tags'] = attr

        logger.debug('BUILD %s FROM %s', tmp['objectID'], self.model)
        return tmp

    def update_obj_index(self, instance):
        '''Update the object.'''
        if self.should_index:
            attr = getattr(instance, self.should_index)
            if not attr():
                # Should not index, but since we don't now the state of the
                # instance, we need to send a DELETE request to ensure that if
                # the instance was previously indexed, it will be removed.
                self.delete_obj_index(self, instance)
                return

        obj = self._build_object(instance)
        self.__index.save_object(obj)
        logger.debug('UPDATE %s FROM %s', obj['objectID'], self.model)

    def delete_obj_index(self, instance):
        '''Delete the object.'''
        objectID = self.__get_objectID(instance)
        self.__index.delete_object(objectID)
        logger.debug('DELETE %s FROM %s', objectID, self.model)

    def raw_search(self, query='', params={}):
        return self.__index.search(query, params)

    def set_settings(self):
        '''Apply the settings to the index.'''
        if self.settings:
            self.__index.set_settings(self.settings)
            logger.debug('APPLY SETTINGS ON %s', self.index_name)

    def clear_index(self):
        '''Clear the index.'''
        self.__index.clear_index()
        logger.debug('CLEAR INDEX %s', self.index_name)

    def reindex_all(self, batch_size=1000):
        '''Reindex all records.'''
        if self.settings:
            self.__tmp_index.set_settings(self.settings)
            logger.debug('APPLY SETTINGS ON %s_tmp', self.index_name)
        self.__tmp_index.clear_index()
        logger.debug('CLEAR INDEX %s_tmp', self.index_name)

        result = None
        counts = 0
        batch = []
        for instance in self.model.objects.all():
            if self.should_index:
                attr = getattr(instance, self.should_index)
                if not attr():
                    continue # should not index

            batch.append(self._build_object(instance))
            if len(batch) >= batch_size:
                result = self.__tmp_index.save_objects(batch)
                logger.info('SAVE %d OBJECTS TO %s_tmp', len(batch),
                            self.index_name)
                batch = []
            counts += 1
        if len(batch) > 0:
            result = self.__tmp_index.save_objects(batch)
            logger.info('SAVE %d OBJECTS TO %s_tmp', len(batch),
                        self.index_name)
        if result:
            self.__client.move_index(self.index_name + '_tmp', self.index_name)
            logger.info('MOVE INDEX %s_tmp TO %s', self.index_name,
                        self.index_name)
        return counts



#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

path_readme = os.path.join(os.path.dirname(__file__), 'README.md')
try:
    import pypandoc
    README = pypandoc.convert(path_readme, 'rst')
except (IOError, ImportError):
    with open(path_readme) as readme:
        README = readme.read()

path_version = os.path.join(os.path.dirname(__file__), 'src/version.py')
if sys.version_info[0] == 3:
    exec(open(path_version).read())
else:
    execfile(path_version)


setup(
    name = 'algoliasearch-django',
    version = VERSION,
    license = 'MIT License',
    packages = [
        'django.contrib.algoliasearch',
        'django.contrib.algoliasearch.management',
        'django.contrib.algoliasearch.management.commands'
    ],
    package_dir = {
        'django.contrib.algoliasearch': 'src',
        'django.contrib.algoliasearch.management': 'src/management',
        'django.contrib.algoliasearch.management.commands': 'src/management/commands'
    },
    install_requires = ['django>=1.7', 'algoliasearch'],
    description = 'Algolia Search integration for Django',
    long_description = README,
    author = 'Algolia Team',
    author_email = 'support@algolia.com',
    url = 'https://github.com/algolia/algoliasearch-django',
    keywords = ['algolia', 'pyalgolia', 'search', 'backend', 'hosted', 'cloud',
        'full-text search', 'faceted search', 'django'],
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
